<?php

namespace Litecms\Account\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Account\Forms\Journal as JournalForm;
use Litecms\Account\Http\Requests\JournalResourceRequest;
use Litecms\Account\Http\Resources\JournalResource;
use Litecms\Account\Http\Resources\JournalsCollection;
use Litecms\Account\Models\Journal;
use Litecms\Account\Actions\JournalAction;
use Litecms\Account\Actions\JournalActions;


/**
 * Resource controller class for journal.
 */
class JournalResourceController extends BaseController
{

    /**
     * Initialize journal resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = JournalForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.account.modules'), 'account', guard_url('account'));
            return $next($request);
        });
    }

    /**
     * Display a list of journal.
     *
     * @return Response
     */
    public function index(JournalResourceRequest $request)
    {
        $request = $request->all();
        $page = JournalActions::run('paginate', $request);

        $data = new JournalsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('account::journal.names'))
            ->view('account::journal.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display journal.
     *
     * @param Request $request
     * @param Model   $journal
     *
     * @return Response
     */
    public function show(JournalResourceRequest $request, Journal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new JournalResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('account::journal.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('account::journal.show')
            ->output();
    }

    /**
     * Show the form for creating a new journal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(JournalResourceRequest $request, Journal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new JournalResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::journal.name'))
            ->view('account::journal.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new journal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(JournalResourceRequest $request, Journal $model)
    {
        try {
            $request = $request->all();
            $model = JournalAction::run('store', $model, $request);
            $data = new JournalResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::journal.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/journal/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/journal'))
                ->redirect();
        }

    }

    /**
     * Show journal for editing.
     *
     * @param Request $request
     * @param Model   $journal
     *
     * @return Response
     */
    public function edit(JournalResourceRequest $request, Journal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new JournalResource($model);
        // return view('account::journal.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::journal.name'))
            ->view('account::journal.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the journal.
     *
     * @param Request $request
     * @param Model   $journal
     *
     * @return Response
     */
    public function update(JournalResourceRequest $request, Journal $model)
    {
        try {
            $request = $request->all();
            $model = JournalAction::run('update', $model, $request);
            $data = new JournalResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::journal.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/journal/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/journal/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the journal.
     *
     * @param Model   $journal
     *
     * @return Response
     */
    public function destroy(JournalResourceRequest $request, Journal $model)
    {
        try {

            $request = $request->all();
            $model = JournalAction::run('destroy', $model, $request);
            $data = new JournalResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::journal.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/journal/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/journal/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
