<?php

namespace Litecms\Account\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Account\Forms\Ledger as LedgerForm;
use Litecms\Account\Http\Requests\LedgerResourceRequest;
use Litecms\Account\Http\Resources\LedgerResource;
use Litecms\Account\Http\Resources\LedgersCollection;
use Litecms\Account\Models\Ledger;
use Litecms\Account\Actions\LedgerAction;
use Litecms\Account\Actions\LedgerActions;


/**
 * Resource controller class for ledger.
 */
class LedgerResourceController extends BaseController
{

    /**
     * Initialize ledger resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = LedgerForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.account.modules'), 'account', guard_url('account'));
            return $next($request);
        });
    }

    /**
     * Display a list of ledger.
     *
     * @return Response
     */
    public function index(LedgerResourceRequest $request)
    {
        $request = $request->all();
        $page = LedgerActions::run('paginate', $request);

        $data = new LedgersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('account::ledger.names'))
            ->view('account::ledger.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display ledger.
     *
     * @param Request $request
     * @param Model   $ledger
     *
     * @return Response
     */
    public function show(LedgerResourceRequest $request, Ledger $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LedgerResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('account::ledger.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('account::ledger.show')
            ->output();
    }

    /**
     * Show the form for creating a new ledger.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LedgerResourceRequest $request, Ledger $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LedgerResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::ledger.name'))
            ->view('account::ledger.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new ledger.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LedgerResourceRequest $request, Ledger $model)
    {
        try {
            $request = $request->all();
            $model = LedgerAction::run('store', $model, $request);
            $data = new LedgerResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::ledger.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/ledger/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/ledger'))
                ->redirect();
        }

    }

    /**
     * Show ledger for editing.
     *
     * @param Request $request
     * @param Model   $ledger
     *
     * @return Response
     */
    public function edit(LedgerResourceRequest $request, Ledger $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LedgerResource($model);
        // return view('account::ledger.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::ledger.name'))
            ->view('account::ledger.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the ledger.
     *
     * @param Request $request
     * @param Model   $ledger
     *
     * @return Response
     */
    public function update(LedgerResourceRequest $request, Ledger $model)
    {
        try {
            $request = $request->all();
            $model = LedgerAction::run('update', $model, $request);
            $data = new LedgerResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::ledger.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/ledger/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/ledger/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the ledger.
     *
     * @param Model   $ledger
     *
     * @return Response
     */
    public function destroy(LedgerResourceRequest $request, Ledger $model)
    {
        try {

            $request = $request->all();
            $model = LedgerAction::run('destroy', $model, $request);
            $data = new LedgerResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::ledger.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/ledger/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/ledger/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
