<?php

namespace Litecms\Account\Http\Controllers;

use Exception;
use Litecms\Account\Actions\LedgerWorkflow;
use Litecms\Account\Http\Requests\LedgerWorkflowRequest;
use Litecms\Account\Http\Resources\LedgerResource;
use Litecms\Account\Models\Ledger;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class LedgerWorkflowController extends BaseController
{
    /**
     * Action controller function for ledger.
     *
     * @param Model $ledger
     * @param action next action for the ledger.
     *
     * @return Response
     */
    public function __invoke(LedgerWorkflowRequest $request, Ledger $ledger, $trasition)
    {
        try {
            $request = $request->all();
            $ledger = LedgerWorkflow::run($trasition, $ledger, $request);
            $data = new LedgerResource($ledger);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::ledger.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/ledger/' . $ledger->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/ledger/' . $ledger->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the ledger and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $ledger
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $ledger, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $ledger = Ledger::findBySignedId($ledger);
        $transitions = $this->transitions($ledger, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('account::ledger.name') . ' ' . trans('Approvals'))
            ->view('account::public.ledger.workflow')
            ->layout('mobile')
            ->data(compact('ledger', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the ledger.
     *
     * @param PublicRequest $request
     * @param string $ledger
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $ledger, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $ledger = Ledger::findBySignedId($ledger);
        $user = user();
        $ledger = LedgerWorkflow::run($transition, $ledger, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($ledger, $user)
    {
        $transitions = [];
        foreach ($ledger->workflow()->transitions($ledger) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.ledger.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'ledger' => $ledger->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $ledger->workflow()->form($value);
            $array['label'] = trans('account::ledger.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
