<?php

namespace Litecms\Account\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Account\Forms\Receipt as ReceiptForm;
use Litecms\Account\Http\Requests\ReceiptResourceRequest;
use Litecms\Account\Http\Resources\ReceiptResource;
use Litecms\Account\Http\Resources\ReceiptsCollection;
use Litecms\Account\Models\Receipt;
use Litecms\Account\Actions\ReceiptAction;
use Litecms\Account\Actions\ReceiptActions;


/**
 * Resource controller class for receipt.
 */
class ReceiptResourceController extends BaseController
{

    /**
     * Initialize receipt resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = ReceiptForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.account.modules'), 'account', guard_url('account'));
            return $next($request);
        });
    }

    /**
     * Display a list of receipt.
     *
     * @return Response
     */
    public function index(ReceiptResourceRequest $request)
    {
        $request = $request->all();
        $page = ReceiptActions::run('paginate', $request);

        $data = new ReceiptsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('account::receipt.names'))
            ->view('account::receipt.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display receipt.
     *
     * @param Request $request
     * @param Model   $receipt
     *
     * @return Response
     */
    public function show(ReceiptResourceRequest $request, Receipt $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ReceiptResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('account::receipt.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('account::receipt.show')
            ->output();
    }

    /**
     * Show the form for creating a new receipt.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReceiptResourceRequest $request, Receipt $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ReceiptResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::receipt.name'))
            ->view('account::receipt.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new receipt.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReceiptResourceRequest $request, Receipt $model)
    {
        try {
            $request = $request->all();
            $model = ReceiptAction::run('store', $model, $request);
            $data = new ReceiptResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::receipt.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/receipt/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/receipt'))
                ->redirect();
        }

    }

    /**
     * Show receipt for editing.
     *
     * @param Request $request
     * @param Model   $receipt
     *
     * @return Response
     */
    public function edit(ReceiptResourceRequest $request, Receipt $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ReceiptResource($model);
        // return view('account::receipt.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::receipt.name'))
            ->view('account::receipt.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the receipt.
     *
     * @param Request $request
     * @param Model   $receipt
     *
     * @return Response
     */
    public function update(ReceiptResourceRequest $request, Receipt $model)
    {
        try {
            $request = $request->all();
            $model = ReceiptAction::run('update', $model, $request);
            $data = new ReceiptResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::receipt.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/receipt/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/receipt/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the receipt.
     *
     * @param Model   $receipt
     *
     * @return Response
     */
    public function destroy(ReceiptResourceRequest $request, Receipt $model)
    {
        try {

            $request = $request->all();
            $model = ReceiptAction::run('destroy', $model, $request);
            $data = new ReceiptResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::receipt.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/receipt/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/receipt/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
