<?php

namespace Litecms\Account\Http\Controllers;

use Exception;
use Litecms\Account\Actions\ReceiptWorkflow;
use Litecms\Account\Http\Requests\ReceiptWorkflowRequest;
use Litecms\Account\Http\Resources\ReceiptResource;
use Litecms\Account\Models\Receipt;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class ReceiptWorkflowController extends BaseController
{
    /**
     * Action controller function for receipt.
     *
     * @param Model $receipt
     * @param action next action for the receipt.
     *
     * @return Response
     */
    public function __invoke(ReceiptWorkflowRequest $request, Receipt $receipt, $trasition)
    {
        try {
            $request = $request->all();
            $receipt = ReceiptWorkflow::run($trasition, $receipt, $request);
            $data = new ReceiptResource($receipt);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::receipt.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/receipt/' . $receipt->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/receipt/' . $receipt->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the receipt and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $receipt
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $receipt, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $receipt = Receipt::findBySignedId($receipt);
        $transitions = $this->transitions($receipt, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('account::receipt.name') . ' ' . trans('Approvals'))
            ->view('account::public.receipt.workflow')
            ->layout('mobile')
            ->data(compact('receipt', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the receipt.
     *
     * @param PublicRequest $request
     * @param string $receipt
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $receipt, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $receipt = Receipt::findBySignedId($receipt);
        $user = user();
        $receipt = ReceiptWorkflow::run($transition, $receipt, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($receipt, $user)
    {
        $transitions = [];
        foreach ($receipt->workflow()->transitions($receipt) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.receipt.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'receipt' => $receipt->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $receipt->workflow()->form($value);
            $array['label'] = trans('account::receipt.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
